==> akonadi-1.11.80.tar.bz2: akonadi-1.11.80/server/src/search/searchmanager.h <==
/*
    Copyright (c) 2010 Volker Krause <vkrause@kde.org>
    Copyright (c) 2013 Daniel Vrátil <dvratil@redhat.com>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#ifndef SEARCHMANAGER_H
#define SEARCHMANAGER_H

#include <QObject>
#include <QVector>
#include <QDBusConnection>

#include <libs/notificationmessagev3_p.h>

class QTimer;

namespace Akonadi {

class AbstractSearchPlugin;

namespace Server {

class NotificationCollector;
class AbstractSearchEngine;
class Collection;

/**
 * SearchManager creates and deletes persistent searches for all currently
 * active search engines.
 */
class SearchManager : public QObject
{
  Q_OBJECT
  Q_CLASSINFO( "D-Bus Interface", "org.freedesktop.Akonadi.SearchManager" )

  public:
    /** Create a new search manager with the given @p searchEngines. */
    explicit SearchManager( const QStringList &searchEngines, QObject *parent = 0 );
    ~SearchManager();

    /**
     * Returns a global instance of the search manager.
     */
    static SearchManager *instance();

    /**
     * This is called via D-Bus from AgentManager to register an agent with
     * search interface.
     */
    void registerInstance( const QString &id );

    /**
     * This is called via D-Bus from AgentManager to unregister an agent with
     * search interface.
     */
    void unregisterInstance( const QString &id );

    /**
     * Updates the search query asynchronously. Returns immediately
     */
    void updateSearchAsync( const Collection &collection, NotificationCollector *collector );

    /**
     * Returns currently available search plugins.
     */
    QVector<AbstractSearchPlugin *> searchPlugins() const;

  public Q_SLOTS:
    /**
     * Update the search query of the given collection synchronously.
     *
     * This method blocks until all search queries are finished.
     */
    bool updateSearch( const Collection &collection, NotificationCollector *collector );


  private Q_SLOTS:
    void scheduleSearchUpdate( const Akonadi::NotificationMessageV3::List &notifications );
    void searchUpdateTimeout();
    void searchUpdateResultsAvailable( const QSet<qint64> &results );

  private:
    void loadSearchPlugins();

    static SearchManager *sInstance;

    QVector<AbstractSearchEngine *> mEngines;
    QVector<AbstractSearchPlugin *> mPlugins;

    // agents dbus interface cache
    QDBusConnection mDBusConnection;

    QList<qint64> mPendingSearchUpdateIds;
    QTimer *mSearchUpdateTimer;
};

} // namespace Server
} // namespace Akonadi

#endif

==> akonadi_1.12.1.orig.tar.bz2: akonadi-1.12.1/server/src/search/searchmanager.h <==
/*
    Copyright (c) 2010 Volker Krause <vkrause@kde.org>
    Copyright (c) 2013 Daniel Vrátil <dvratil@redhat.com>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#ifndef SEARCHMANAGER_H
#define SEARCHMANAGER_H

#include <QThread>
#include <QVector>
#include <QMutex>
#include <QDBusConnection>

#include <libs/notificationmessagev3_p.h>

class QWaitCondition;
class QTimer;

namespace Akonadi {

class AbstractSearchPlugin;

namespace Server {

class NotificationCollector;
class AbstractSearchEngine;
class Collection;


class SearchManagerThread : public QThread
{
  public:
    SearchManagerThread( const QStringList &searchEngines, QObject *parent = 0 );
    ~SearchManagerThread();

    void run();

  private:
    QStringList mSearchEngines;
};


/**
 * SearchManager creates and deletes persistent searches for all currently
 * active search engines.
 */
class SearchManager : public QObject
{
  Q_OBJECT
  Q_CLASSINFO( "D-Bus Interface", "org.freedesktop.Akonadi.SearchManager" )

  public:
    /** Create a new search manager with the given @p searchEngines. */
    explicit SearchManager( const QStringList &searchEngines, QObject *parent = 0 );
    ~SearchManager();

    /**
     * Returns a global instance of the search manager.
     */
    static SearchManager *instance();

    /**
     * This is called via D-Bus from AgentManager to register an agent with
     * search interface.
     */
    void registerInstance( const QString &id );

    /**
     * This is called via D-Bus from AgentManager to unregister an agent with
     * search interface.
     */
    void unregisterInstance( const QString &id );

    /**
     * Updates the search query asynchronously. Returns immediately
     */
    void updateSearchAsync( const Collection &collection );

    /**
     * Updates the search query synchronously.
     */
    void updateSearch( const Collection &collection );

    /**
     * Returns currently available search plugins.
     */
    QVector<AbstractSearchPlugin *> searchPlugins() const;

  public Q_SLOTS:
    void scheduleSearchUpdate();

  private Q_SLOTS:
    void searchUpdateTimeout();
    void searchUpdateResultsAvailable( const QSet<qint64> &results );

    /**
     * Actual implementation of search updates.
     *
     * Since caller invokes this method from a different thread, they use
     * QMetaObject::invokeMethod(). To still make it possible for callers to behave
     * synchrounously, we can pass in a QWaitCondition that the code will wake up
     * once the search update is completed.
     */
    void updateSearchImpl( const Collection &collection, QWaitCondition *cond );

  private:
    void loadSearchPlugins();

    static SearchManager *sInstance;

    QVector<AbstractSearchEngine *> mEngines;
    QVector<AbstractSearchPlugin *> mPlugins;

    QTimer *mSearchUpdateTimer;

    QMutex mLock;
    QSet<qint64> mUpdatingCollections;
};

} // namespace Server
} // namespace Akonadi

#endif
